﻿
// ButtonDlg.cpp : 実装ファイル
//

#include "pch.h"
#include "framework.h"
#include "Button.h"
#include "ButtonDlg.h"
#include "afxdialogex.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CButtonDlg ダイアログ



CButtonDlg::CButtonDlg(CWnd* pParent /*=nullptr*/)
	: CDialogEx(IDD_BUTTON_DIALOG, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_uEvent = 0;
}

CButtonDlg::~CButtonDlg()
{
	if (m_uEvent == 1) KillTimer(1);
	if (USBDevice) {	// 作成したUSB通信用のクラスを削除します
		USBDevice->Close();
		delete USBDevice;
		USBDevice = NULL;
	}
}

void CButtonDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CButtonDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_RESET_BUTTON, &CButtonDlg::OnBnClickedResetButton)
	ON_WM_TIMER()
END_MESSAGE_MAP()


// CButtonDlg メッセージ ハンドラー

BOOL CButtonDlg::OnInitDialog()
{
	int i, n;

	CDialogEx::OnInitDialog();

	// このダイアログのアイコンを設定します。アプリケーションのメイン ウィンドウがダイアログでない場合、
	//  Framework は、この設定を自動的に行います。
	SetIcon(m_hIcon, TRUE);			// 大きいアイコンの設定
	SetIcon(m_hIcon, FALSE);		// 小さいアイコンの設定

	USBDevice = new CCyUSBDevice(m_hWnd);	// USB通信用のクラスを生成します
	if (!(n = USBDevice->DeviceCount())) {	// USBケーブルにUSB機器が接続されていることを確認します　
		AfxMessageBox(L"No CyUSB3.sys-drived device is found.");
		PostMessage(WM_CLOSE);
		return TRUE;
	}
	for (i = 0; i < n; i++) {		// 接続されたUSB機器がsimpleio.iicが書き込まれたEZ-USBであることを確認します
		USBDevice->Open(i);
		if (USBDevice->VendorID == 0x04B4 && USBDevice->ProductID == 0x1004) break;
		USBDevice->Close();
	}
	if (i == n) {
		AfxMessageBox(L"No EZ-USB FX2LP Example Device is found.");
		PostMessage(WM_CLOSE);
		return TRUE;
	}
	if (!(n = USBDevice->EndPointCount())) {	// EZ-USBが内包するEndPointの数を確認します
		AfxMessageBox(L"No EndPoint is found.");
		PostMessage(WM_CLOSE);
		return TRUE;
	}
	for (i = 0; i < n; i++) {		// EndPoint 02/86をそれぞれ出力用/入力用変数(OutEndpt/InEndpt)に割り当てます
		if (USBDevice->EndPoints[i]->Address == 0x02) OutEndpt = USBDevice->EndPoints[i];
		else {
			if (USBDevice->EndPoints[i]->Address == 0x86) InEndpt = USBDevice->EndPoints[i];
		}
	}
	m_strDeviceName = USBDevice->FriendlyName;
	SetWindowText(m_strDeviceName);	// 接続されたUSB機器の名称をウィンドウタイトルバーに表示します

	SetDirection(0x00, 0x00, 0x01);	// EZ-USBのPD0のみを出力、他を入力に設定します
	OnBnClickedResetButton();		// 早押しボタンをリセットします
	m_uEvent = 1;
	SetTimer(m_uEvent, 100, NULL);		// 100ミリ秒ごとにOnTimer関数で早押しボタンの状態をチェックします
	return TRUE;  // フォーカスをコントロールに設定した場合を除き、TRUE を返します。
}

// ダイアログに最小化ボタンを追加する場合、アイコンを描画するための
//  下のコードが必要です。ドキュメント/ビュー モデルを使う MFC アプリケーションの場合、
//  これは、Framework によって自動的に設定されます。

void CButtonDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // 描画のデバイス コンテキスト

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// クライアントの四角形領域内の中央
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// アイコンの描画
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// ユーザーが最小化したウィンドウをドラッグしているときに表示するカーソルを取得するために、
//  システムがこの関数を呼び出します。
HCURSOR CButtonDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CButtonDlg::SetDirection(BYTE pa, BYTE pb, BYTE pd)
// simpleio.iicの機能：SetDirectionを用いて各ポートの入出力方向を一度に設定します
{
	LONG len = 4;

	OutBuff[0] = 0;
	OutBuff[1] = pa; OutBuff[2] = pb; OutBuff[3] = pd;
	if (!OutEndpt->XferData(OutBuff, len)) {
		AfxMessageBox(L"USB Bulk Transfer Failed.");
		PostMessage(WM_CLOSE);
	}
}

void CButtonDlg::OnBnClickedResetButton()
// simpleio.iicの機能：OutputPortを用いてPD0に0→1と出力し、早押しボタンをリセットします
{
	LONG len = 2;

	OutBuff[0] = 0x23;
	OutBuff[1] = 0x00;
	if (!OutEndpt->XferData(OutBuff, len)) {
		AfxMessageBox(L"USB Bulk Transfer Failed.");
		PostMessage(WM_CLOSE);
		return;
	}
	OutBuff[1] = 0x01;
	if (!OutEndpt->XferData(OutBuff, len)) {
		AfxMessageBox(L"USB Bulk Transfer Failed.");
		PostMessage(WM_CLOSE);
	}

	GetDlgItem(IDC_RESULT_STATIC)->SetWindowText(_T(""));	//　結果表示のクリア
}


void CButtonDlg::OnTimer(UINT_PTR nIDEvent)
// simpleio.iicの機能：ReadPortsを用いて早押しボタンの結果を読んで、結果を表示します
{
	LONG len = 1;

	if (nIDEvent == m_uEvent) {
		OutBuff[0] = 0x10;
		if (!OutEndpt->XferData(OutBuff, len)) {
			AfxMessageBox(L"USB Bulk Transfer Failed.");
			PostMessage(WM_CLOSE);
			return;
		}
		len = 3;
		if (!InEndpt->XferData(InBuff, len)) {
			AfxMessageBox(L"USB Bulk Transfer Failed.");
			PostMessage(WM_CLOSE);
			return;
		}
		switch (InBuff[0] & 3) {
		case 0:GetDlgItem(IDC_RESULT_STATIC)->SetWindowText(_T("回路が壊れました")); break;
		case 1:GetDlgItem(IDC_RESULT_STATIC)->SetWindowText(_T("ボタンBが押されました")); break;
		case 2:GetDlgItem(IDC_RESULT_STATIC)->SetWindowText(_T("ボタンAが押されました")); break;
		case 3:GetDlgItem(IDC_RESULT_STATIC)->SetWindowText(_T("")); break;
		}
	}
	CDialogEx::OnTimer(nIDEvent);
}
